@@ -23,6 +23,8 @@
  * This file implements gcrypt support functions in vlc
  */
 
+#include <errno.h>
+
 #ifdef LIBVLC_USE_PTHREAD
 /**
  * If possible, use gcrypt-provided thread implementation. This is so that
@@ -38,18 +40,13 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 
 static int gcry_vlc_mutex_init( void **p_sys )
 {
-    int i_val;
     vlc_mutex_t *p_lock = (vlc_mutex_t *)malloc( sizeof( vlc_mutex_t ) );
-
     if( p_lock == NULL)
         return ENOMEM;
 
-    i_val = vlc_mutex_init( p_lock );
-    if( i_val )
-        free( p_lock );
-    else
-        *p_sys = p_lock;
-    return i_val;
+    vlc_mutex_init( p_lock );
+    *p_sys = p_lock;
+    return VLC_SUCCESS;
 }
 
 static int gcry_vlc_mutex_destroy( void **p_sys )
@@ -79,7 +76,8 @@ static const struct gcry_thread_cbs gcry_threads_vlc =
     gcry_vlc_mutex_init,
     gcry_vlc_mutex_destroy,
     gcry_vlc_mutex_lock,
-    gcry_vlc_mutex_unlock
+    gcry_vlc_mutex_unlock,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
 };
 #endif
 
@@ -88,7 +86,19 @@ static const struct gcry_thread_cbs gcry_threads_vlc =
  */
 static inline void vlc_gcrypt_init (void)
 {
-    vlc_mutex_t *lock = var_AcquireMutex ("gcrypt_mutex");
-    gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_vlc);
-    vlc_mutex_unlock (lock);
+    /* This would need a process-wide static mutex with all libraries linking
+     * to a given instance of libgcrypt. We cannot do this as we have different
+     * plugins linking with gcrypt, and some underlying libraries may use it
+     * behind our back. Only way is to always link gcrypt statically (ouch!) or
+     * have upstream gcrypt provide one shared object per threading system. */
+    static vlc_mutex_t lock = VLC_STATIC_MUTEX;
+    static bool done = false;
+
+    vlc_mutex_lock (&lock);
+    if (!done)
+    {
+        gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_vlc);
+        done = true;
+    }
+    vlc_mutex_unlock (&lock);
 }
