@@ -2,7 +2,7 @@
  * EPGView.h : EPGView
  ****************************************************************************
  * Copyright © 2009-2010 VideoLAN
- * $Id: 2fd66fc1465fc793fb812606b80ea38bf3287450 $
+ * $Id: 6c26be11f202a0409c2a988a95b1e56acd3bf61c $
  *
  * Authors: Ludovic Fauvet <etix@l0cal.com>
  *
@@ -29,7 +29,7 @@
 #include <QGraphicsView>
 #include <QList>
 
-#define TRACKS_HEIGHT 75
+#define TRACKS_HEIGHT 60
 
 class QDateTime;
 class EPGView : public QGraphicsView
@@ -41,20 +41,20 @@ public:
 
     void            setScale( double scaleFactor );
 
-    void            setStartTime( const QDateTime& startTime );
+    void            updateStartTime();
     const QDateTime& startTime();
 
     void            addEvent( EPGEvent* event );
-    void            updateEvent( EPGEvent* event );
     void            delEvent( EPGEvent* event );
     void            updateDuration();
 
+    QList<QString>  getChannelList();
+
 signals:
     void            startTimeChanged( const QDateTime& startTime );
     void            durationChanged( int seconds );
     void            eventFocusedChanged( EPGEvent * );
 protected:
-    virtual void    drawBackground( QPainter *painter, const QRectF &rect );
 
     QList<QString>  m_channels;
     QDateTime       m_startTime;
