@@ -2,7 +2,7 @@
  * input_manager.cpp : Manage an input and interact with its GUI elements
  ****************************************************************************
  * Copyright (C) 2006-2008 the VideoLAN team
- * $Id: 7961729f7b8e8aa954bac3615add3616df00934e $
+ * $Id: a464a4e5e43771b5b1e3445c038559ed6d00a703 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Ilkka Ollakka  <ileoo@videolan.org>
@@ -862,7 +862,7 @@ void InputManager::setRate( int new_rate )
 void InputManager::jumpFwd()
 {
     int i_interval = var_InheritInteger( p_input, "short-jump-size" );
-    if( i_interval > 0 )
+    if( i_interval > 0 && hasInput() )
     {
         mtime_t val = (mtime_t)(i_interval) * 1000000L;
         var_SetTime( p_input, "time-offset", val );
@@ -872,7 +872,7 @@ void InputManager::jumpFwd()
 void InputManager::jumpBwd()
 {
     int i_interval = var_InheritInteger( p_input, "short-jump-size" );
-    if( i_interval > 0 )
+    if( i_interval > 0 && hasInput() )
     {
         mtime_t val = -1 *(mtime_t)(i_interval) * 1000000L;
         var_SetTime( p_input, "time-offset", val );
