@@ -1,8 +1,8 @@
 /*****************************************************************************
  * visual.c : Visualisation system
  *****************************************************************************
- * Copyright (C) 2002-2006 the VideoLAN team
- * $Id: 73a71a67217e56fcee4d674c003569e2c05f309e $
+ * Copyright (C) 2002-2009 the VideoLAN team
+ * $Id: cf3c00ff5255b30f057c687ff3cdfe4e8eb2d390 $
  *
  * Authors: Clément Stenac <zorglub@via.ecp.fr>
  *
@@ -32,6 +32,7 @@
 #include <vlc_plugin.h>
 #include <vlc_vout.h>
 #include <vlc_aout.h>
+#include <vlc_filter.h>
 
 #include "visual.h"
 
@@ -41,7 +42,8 @@
 #define ELIST_TEXT N_( "Effects list" )
 #define ELIST_LONGTEXT N_( \
       "A list of visual effect, separated by commas.\n"  \
-      "Current effects include: dummy, scope, spectrum." )
+      "Current effects include: dummy, scope, spectrum, "\
+      "spectrometer and vuMeter." )
 
 #define WIDTH_TEXT N_( "Video width" )
 #define WIDTH_LONGTEXT N_( \
@@ -51,11 +53,11 @@
 #define HEIGHT_LONGTEXT N_( \
       "The height of the effects video window, in pixels." )
 
-#define NBBANDS_TEXT N_( "Number of bands" )
+#define NBBANDS_TEXT N_( "More bands : 80 / 20" )
 #define NBBANDS_LONGTEXT N_( \
-      "Number of bands used by spectrum analyzer, should be 20 or 80." )
+      "More bands for the spectrum analyzer : 80 if enabled else 20." )
 #define SPNBBANDS_LONGTEXT N_( \
-      "Number of bands used by the spectrometer, from 20 to 80." )
+      "More bands for the spectrometer : 80 if enabled else 20." )
 
 #define SEPAR_TEXT N_( "Band separator" )
 #define SEPAR_LONGTEXT N_( \
@@ -101,85 +103,80 @@
 #define COLOR1_LONGTEXT N_( \
         "YUV-Color cube shifting across the V-plane ( 0 - 127 )." )
 
-#define STARS_TEXT N_( "Number of stars" )
-#define STARS_LONGTEXT N_( \
-        "Number of stars to draw with random effect." )
-
 static int  Open         ( vlc_object_t * );
 static void Close        ( vlc_object_t * );
 
-vlc_module_begin();
-    set_shortname( N_("Visualizer"));
-    set_category( CAT_AUDIO );
-    set_subcategory( SUBCAT_AUDIO_VISUAL );
-    set_description( N_("Visualizer filter") );
-    set_section( N_( "General") , NULL );
+vlc_module_begin ()
+    set_shortname( N_("Visualizer"))
+    set_category( CAT_AUDIO )
+    set_subcategory( SUBCAT_AUDIO_VISUAL )
+    set_description( N_("Visualizer filter") )
+    set_section( N_( "General") , NULL )
     add_string("effect-list", "spectrum", NULL,
-            ELIST_TEXT, ELIST_LONGTEXT, true );
+            ELIST_TEXT, ELIST_LONGTEXT, true )
     add_integer("effect-width",VOUT_WIDTH,NULL,
-             WIDTH_TEXT, WIDTH_LONGTEXT, false );
+             WIDTH_TEXT, WIDTH_LONGTEXT, false )
     add_integer("effect-height" , VOUT_HEIGHT , NULL,
-             HEIGHT_TEXT, HEIGHT_LONGTEXT, false );
-    set_section( N_("Spectrum analyser") , NULL );
-    add_integer("visual-nbbands", 80, NULL,
+             HEIGHT_TEXT, HEIGHT_LONGTEXT, false )
+    set_section( N_("Spectrum analyser") , NULL )
+    add_obsolete_integer( "visual-nbbands" ) /* Since 1.0.0 */
+    add_bool("visual-80-bands", true, NULL,
              NBBANDS_TEXT, NBBANDS_LONGTEXT, true );
-    add_integer("visual-separ", 1, NULL,
-             SEPAR_TEXT, SEPAR_LONGTEXT, true );
-    add_integer("visual-amp", 3, NULL,
-             AMP_TEXT, AMP_LONGTEXT, true );
+    add_obsolete_integer( "visual-separ" ) /* Since 1.0.0 */
+    add_obsolete_integer( "visual-amp" ) /* Since 1.0.0 */
     add_bool("visual-peaks", true, NULL,
-             PEAKS_TEXT, PEAKS_LONGTEXT, true );
-    set_section( N_("Spectrometer") , NULL );
+             PEAKS_TEXT, PEAKS_LONGTEXT, true )
+    set_section( N_("Spectrometer") , NULL )
     add_bool("spect-show-original", false, NULL,
-             ORIG_TEXT, ORIG_LONGTEXT, true );
+             ORIG_TEXT, ORIG_LONGTEXT, true )
     add_bool("spect-show-base", true, NULL,
-             BASE_TEXT, BASE_LONGTEXT, true );
+             BASE_TEXT, BASE_LONGTEXT, true )
     add_integer("spect-radius", 42, NULL,
-             RADIUS_TEXT, RADIUS_LONGTEXT, true );
+             RADIUS_TEXT, RADIUS_LONGTEXT, true )
     add_integer("spect-sections", 3, NULL,
-             SSECT_TEXT, SSECT_LONGTEXT, true );
+             SSECT_TEXT, SSECT_LONGTEXT, true )
     add_integer("spect-color", 80, NULL,
-             COLOR1_TEXT, COLOR1_LONGTEXT, true );
+             COLOR1_TEXT, COLOR1_LONGTEXT, true )
     add_bool("spect-show-bands", true, NULL,
              BANDS_TEXT, BANDS_LONGTEXT, true );
-    add_integer("spect-nbbands", 32, NULL,
-             NBBANDS_TEXT, SPNBBANDS_LONGTEXT, true );
+    add_obsolete_integer( "spect-nbbands" ) /* Since 1.0.0 */
+    add_bool("spect-80-bands", true, NULL,
+             NBBANDS_TEXT, SPNBBANDS_LONGTEXT, true )
     add_integer("spect-separ", 1, NULL,
-             SEPAR_TEXT, SEPAR_LONGTEXT, true );
+             SEPAR_TEXT, SEPAR_LONGTEXT, true )
     add_integer("spect-amp", 8, NULL,
-             AMP_TEXT, AMP_LONGTEXT, true );
+             AMP_TEXT, AMP_LONGTEXT, true )
     add_bool("spect-show-peaks", true, NULL,
-             PEAKS_TEXT, PEAKS_LONGTEXT, true );
+             PEAKS_TEXT, PEAKS_LONGTEXT, true )
     add_integer("spect-peak-width", 61, NULL,
-             PEAK_WIDTH_TEXT, PEAK_WIDTH_LONGTEXT, true );
+             PEAK_WIDTH_TEXT, PEAK_WIDTH_LONGTEXT, true )
     add_integer("spect-peak-height", 1, NULL,
-             PEAK_HEIGHT_TEXT, PEAK_HEIGHT_LONGTEXT, true );
-    set_capability( "visualization", 0 );
-    set_callbacks( Open, Close );
-    add_shortcut( "visualizer");
-vlc_module_end();
+             PEAK_HEIGHT_TEXT, PEAK_HEIGHT_LONGTEXT, true )
+    set_capability( "visualization2", 0 )
+    set_callbacks( Open, Close )
+    add_shortcut( "visualizer")
+vlc_module_end ()
 
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static void DoWork( aout_instance_t *, aout_filter_t *,
-                    aout_buffer_t *, aout_buffer_t * );
+static block_t *DoWork( filter_t *, block_t * );
 static int FilterCallback( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
 static const struct
 {
     const char *psz_name;
-    int  (*pf_run)( visual_effect_t *, aout_instance_t *,
-                    aout_buffer_t *, picture_t *);
+    int  (*pf_run)( visual_effect_t *, vlc_object_t *,
+                    const block_t *, picture_t *);
 } pf_effect_run[]=
 {
-    { "scope",      scope_Run },
-    { "vuMeter",    vuMeter_Run },
-    { "spectrum",   spectrum_Run },
-    { "spectrometer",   spectrometer_Run },
-    { "dummy",      dummy_Run},
-    { NULL,         dummy_Run}
+    { "scope",        scope_Run },
+    { "vuMeter",      vuMeter_Run },
+    { "spectrum",     spectrum_Run },
+    { "spectrometer", spectrometer_Run },
+    { "dummy",        dummy_Run},
+    { NULL,           dummy_Run}
 };
 
 /*****************************************************************************
@@ -187,29 +184,27 @@ static const struct
  *****************************************************************************/
 static int Open( vlc_object_t *p_this )
 {
-    aout_filter_t     *p_filter = (aout_filter_t *)p_this;
-    aout_filter_sys_t *p_sys;
-    vlc_value_t        val;
+    filter_t     *p_filter = (filter_t *)p_this;
+    filter_sys_t *p_sys;
 
     char *psz_effects, *psz_parser;
     video_format_t fmt;
 
-
-    if( ( p_filter->input.i_format != VLC_FOURCC('f','l','3','2') &&
-          p_filter->input.i_format != VLC_FOURCC('f','i','3','2') ) )
+    if( ( p_filter->fmt_in.audio.i_format != VLC_CODEC_FL32 &&
+          p_filter->fmt_in.audio.i_format != VLC_CODEC_FI32 ) )
     {
         return VLC_EGENERIC;
     }
 
-    p_sys = p_filter->p_sys = malloc( sizeof( aout_filter_sys_t ) );
+    p_sys = p_filter->p_sys = malloc( sizeof( filter_sys_t ) );
     if( p_sys == NULL )
         return VLC_EGENERIC;
 
-    p_sys->i_height = config_GetInt( p_filter , "effect-height");
-    p_sys->i_width  = config_GetInt( p_filter , "effect-width");
+    p_sys->i_height = var_InheritInteger( p_filter , "effect-height");
+    p_sys->i_width  = var_InheritInteger( p_filter , "effect-width");
 
-    if( p_sys->i_height < 20 ) p_sys->i_height =  20;
-    if( p_sys->i_width  < 20 ) p_sys->i_width  =  20;
+    if( p_sys->i_height < 400 ) p_sys->i_height = 400;
+    if( p_sys->i_width  < 532 ) p_sys->i_width  = 532;
     if( (p_sys->i_height % 2 ) != 0 ) p_sys->i_height--;
     if( (p_sys->i_width % 2 )  != 0 ) p_sys->i_width--;
 
@@ -217,11 +212,7 @@ static int Open( vlc_object_t *p_this )
     p_sys->effect   = NULL;
 
     /* Parse the effect list */
-    var_Create( p_filter, "effect-list", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
-    var_Get( p_filter, "effect-list", &val);
-    psz_parser = psz_effects = strdup( val.psz_string );
-    free( val.psz_string );
-
+    psz_parser = psz_effects = var_CreateGetString( p_filter, "effect-list" );
     var_AddCallback( p_filter, "effect-list", FilterCallback, NULL );
 
     while( psz_parser && *psz_parser != '\0' )
@@ -234,7 +225,10 @@ static int Open( vlc_object_t *p_this )
             break;
         p_effect->i_width = p_sys->i_width;
         p_effect->i_height= p_sys->i_height;
-        p_effect->i_nb_chans = aout_FormatNbChannels( &p_filter->input);
+        p_effect->i_nb_chans = aout_FormatNbChannels( &p_filter->fmt_in.audio);
+        p_effect->i_idx_left  = 0;
+        p_effect->i_idx_right = __MIN( 1, p_effect->i_nb_chans-1 );
+
         p_effect->psz_args = NULL;
         p_effect->p_data = NULL;
 
@@ -248,7 +242,7 @@ static int Open( vlc_object_t *p_this )
                               strlen( pf_effect_run[i].psz_name ) ) )
             {
                 p_effect->pf_run = pf_effect_run[i].pf_run;
-                p_effect->psz_name = strdup( pf_effect_run[i].psz_name );
+                p_effect->psz_name = pf_effect_run[i].psz_name;
                 break;
             }
         }
@@ -266,6 +260,7 @@ static int Open( vlc_object_t *p_this )
                 if( ( psz_eoa = strchr( psz_parser, '}') ) == NULL )
                 {
                    msg_Err( p_filter, "unable to parse effect list. Aborting");
+                   free( p_effect );
                    break;
                 }
                 p_effect->psz_args =
@@ -307,20 +302,24 @@ static int Open( vlc_object_t *p_this )
 
     fmt.i_width = fmt.i_visible_width = p_sys->i_width;
     fmt.i_height = fmt.i_visible_height = p_sys->i_height;
-    fmt.i_chroma = VLC_FOURCC('I','4','2','0');
-    fmt.i_aspect = VOUT_ASPECT_FACTOR * p_sys->i_width/p_sys->i_height;
+    fmt.i_chroma = VLC_CODEC_I420;
     fmt.i_sar_num = fmt.i_sar_den = 1;
 
-    p_sys->p_vout = vout_Request( p_filter, NULL, &fmt );
+    p_sys->p_vout = aout_filter_RequestVout( p_filter, NULL, &fmt );
     if( p_sys->p_vout == NULL )
     {
         msg_Err( p_filter, "no suitable vout module" );
+        for( int i = 0; i < p_sys->i_effect; i++ )
+        {
+            free( p_sys->effect[i]->psz_args );
+            free( p_sys->effect[i] );
+        }
+        free( p_sys->effect );
         free( p_sys );
         return VLC_EGENERIC;
     }
 
-    p_filter->pf_do_work = DoWork;
-    p_filter->b_in_place= 1;
+    p_filter->pf_audio_filter = DoWork;
 
     return VLC_SUCCESS;
 }
@@ -330,25 +329,18 @@ static int Open( vlc_object_t *p_this )
  *****************************************************************************
  * Audio part pasted from trivial.c
  ****************************************************************************/
-static void DoWork( aout_instance_t *p_aout, aout_filter_t *p_filter,
-                    aout_buffer_t *p_in_buf, aout_buffer_t *p_out_buf )
+static block_t *DoWork( filter_t *p_filter, block_t *p_in_buf )
 {
-    aout_filter_sys_t *p_sys = p_filter->p_sys;
+    filter_sys_t *p_sys = p_filter->p_sys;
     picture_t *p_outpic;
     int i;
 
-    p_out_buf->i_nb_samples = p_in_buf->i_nb_samples;
-    p_out_buf->i_nb_bytes = p_in_buf->i_nb_bytes *
-                            aout_FormatNbChannels( &p_filter->output ) /
-                            aout_FormatNbChannels( &p_filter->input );
-
     /* First, get a new picture */
     while( ( p_outpic = vout_CreatePicture( p_sys->p_vout, 0, 0, 3 ) ) == NULL)
-    {
-        if( !vlc_object_alive (p_aout) )
-        {
-            return;
-        }
+    {   /* XXX: This looks like a bad idea. Don't run to me for sympathy if it
+         * dead locks... */
+        if( !vlc_object_alive (p_sys->p_vout) )
+            return NULL;
         msleep( VOUT_OUTMEM_SLEEP );
     }
 
@@ -365,15 +357,16 @@ static void DoWork( aout_instance_t *p_aout, aout_filter_t *p_filter,
 #define p_effect p_sys->effect[i]
         if( p_effect->pf_run )
         {
-            p_effect->pf_run( p_effect, p_aout, p_out_buf, p_outpic );
+            p_effect->pf_run( p_effect, VLC_OBJECT(p_filter),
+                              p_in_buf, p_outpic );
         }
 #undef p_effect
     }
 
-    vout_DatePicture( p_sys->p_vout, p_outpic,
-                      ( p_in_buf->start_date + p_in_buf->end_date ) / 2 );
+    p_outpic->date = p_in_buf->i_pts + (p_in_buf->i_length / 2);
 
     vout_DisplayPicture( p_sys->p_vout, p_outpic );
+    return p_in_buf;
 }
 
 /*****************************************************************************
@@ -381,22 +374,34 @@ static void DoWork( aout_instance_t *p_aout, aout_filter_t *p_filter,
  *****************************************************************************/
 static void Close( vlc_object_t *p_this )
 {
-    aout_filter_t * p_filter = (aout_filter_t *)p_this;
-    aout_filter_sys_t *p_sys = p_filter->p_sys;
+    filter_t * p_filter = (filter_t *)p_this;
+    filter_sys_t *p_sys = p_filter->p_sys;
 
     int i;
 
     if( p_filter->p_sys->p_vout )
     {
-        vout_Request( p_filter, p_filter->p_sys->p_vout, 0 );
+        aout_filter_RequestVout( p_filter, p_filter->p_sys->p_vout, 0 );
     }
 
     /* Free the list */
     for( i = 0; i < p_sys->i_effect; i++ )
     {
 #define p_effect p_sys->effect[i]
+        if( !strncmp( p_effect->psz_name, "spectrum", strlen( "spectrum" ) ) )
+        {
+            spectrum_data *p_data = p_effect->p_data;
+            free( p_data->peaks );
+            free( p_data->prev_heights );
+            free( p_data->p_prev_s16_buff );
+        }
+        if( !strncmp( p_effect->psz_name, "spectrometer", strlen( "spectrometer" ) ) )
+        {
+            spectrometer_data *p_data = p_effect->p_data;
+            free( p_data->peaks );
+            free( p_data->p_prev_s16_buff );
+        }
         free( p_effect->p_data );
-        free( p_effect->psz_name );
         free( p_effect->psz_args );
         free( p_effect );
 #undef p_effect
